import cv2 
import os
from sklearn.model_selection import train_test_split
import numpy as np
import time


class Dataset:
    """
    A class for collecting a dataset of images using OpenCV.

    Attributes:
        index (int): The index of the camera to be used for creating the dataset.
        label (list): The list of names of the different types of images to be collected.
        num (int): The number of frames to shoot for every image type.
        height (int): The height of the individual frames.
        width (int): The width of the individual frames.
        standby_time (float): The number of seconds to wait between every frame.
        perc (int): The percentage of data to use for training (the remaining will be used for validation).
        name (str): The name of the dataset.
    
    Examples:
        >>> import xeye
        >>> data = xeye.Dataset()
        >>> data.setup()
        >>> data.var_control()
        >>> data.preview()
        >>> data.rgb() # or data.gray()
        >>> data.compress_train_test()
        >>> data.compress_all()
        >>> data.just.compress()
    """
    index = 0
    label = []
    num = 0
    height = 0
    width = 0
    _class_dict = {}
    _tensor = {}
    standby_time = 0
    _statusGray = 0
    _statusRGB = 0
    perc = 0
    name = "dataset_raw"


    @classmethod
    def setup(cls) -> None:
        """
        Starts the terminal interface for setting up the parameters of the dataset.

        Returns:
            None
        """
        # clear terminal 
        if(os.name == 'posix'): #unix
            os.system('clear')
        else: #windows
            os.system('cls')
        # camera setting control 
        print('--- CAMERA SETTING ---')
        cls.index = int(input('Select the index of the camera that you want to use for creating the dataset: '))
        if cls.index == -1:
            raise ValueError('Insert valid camera index...')
        camera = cv2.VideoCapture(cls.index)
        if camera.isOpened() == False:
            raise ValueError('Insert valid camera index...')
        camera.release()
        # set how many type of images do you want to collect
        cls.label = []
        print('\n')
        print('--- IMAGE SETTINGS ---')
        img_types = int(input('Num. of types of images to scan: '))
        if img_types == 0: 
            raise ValueError('Number of images types must be greather than 0')
        for i in range(0,img_types):
            l = str(input(f"Name of image type ({i+1}): "))
            cls.label.append(l)
        # folder building
        for lab in cls.label:
            if not os.path.exists(lab):
                os.mkdir(lab)
        # number of frames 
        cls.num = int(input('Num. of frames to shoot for every image type: '))
        if cls.num == 0 or cls.num < 0:
            raise ValueError('You cant inizialize frames number equal or below zero...')
        # Image shaping phase
        cls.height = int(input('Single frame HEIGHT: '))
        if cls.height == 0 or cls.height < 0:
            raise ValueError('Frame HEIGHT must be greather than 0')
        cls.width = int(input('Single frame WIDTH: '))
        if cls.width == 0 or cls.width < 0:
            raise ValueError('Frame WIDTH must be greather than 0')

        # Waiting time in shooting loop
        cls.standby_time = float(input('num. of waiting time (in sec.) between every frame: '))
        if cls.standby_time < 0:
            raise ValueError('waiting time must be grater than 0...')


    @classmethod
    def preview(cls) -> None:
        """
        Opens the camera stream on a window for checking the framing of the image.

        Returns: 
            None
        """
        print('\n')
        print('--- PREVIEW ---')
        camera = cv2.VideoCapture(cls.index)
        while(True):
            status, frame = camera.read()
            if not status:
                print("frame doesn't been captured")
                break
            font = cv2.FONT_HERSHEY_COMPLEX
            text = 'click on the image window and then press [q] on the keyboard to quit preview'
            cv2.putText(frame,text,(5,50),font,0.8,(124,252,0),2)  #text,coordinate,font,size of text,color,thickness of font
            cv2.startWindowThread()
            cv2.imshow("Camera PreView", frame)
            if cv2.waitKey(1) == ord('q'):
                print('preview closed')
                camera.release()
                cv2.destroyAllWindows()
                cv2.waitKey(1)
                break


    @classmethod
    def gray(cls) -> None:
        """
        Method for shooting images in grayscale.

        Returns:
            None 
        """
        print('\n')
        print('--- START TAKING PHOTOS ---')
        camera = cv2.VideoCapture(cls.index)
        # Index for files name 
        i = 0
        for folder in cls.label:
            count = 0
            print(f'Press [b] on the keyboard to start data collection of image type: [{folder}]')
            userinput = input()
            if userinput != 'b':
                print("Wrong Input...press 'b'")
            while count < cls.num:
                status, frame = camera.read()
                if not status:
                    print("frame doesn't been captured")
                    break
                gray = cv2.cvtColor(frame, cv2.COLOR_BGR2GRAY)
                cv2.startWindowThread()
                cv2.imshow(f"Camera View for image type [{folder}]", gray)
                gray = cv2.resize(gray, (cls.width, cls.height))
                cv2.imwrite(folder+'/'+ str(cls.label[i]) + str(count) + '.png', gray)
                count=count+1
                time.sleep(cls.standby_time)
                if cv2.waitKey(1) == ord('q'):
                    break
            i += 1
        camera.release()
        cv2.destroyAllWindows()
        cv2.waitKey(1)
        # set status
        cls._statusGray = 1
        cls._statusRGB = 0


    @classmethod
    def rgb(cls) -> None:
        """
        Method for shooting images in RGB.

        Returns:
            None
        """
        print('\n')
        print('--- START TAKING PHOTOS ---')
        camera = cv2.VideoCapture(cls.index)
        # Index for files name 
        i = 0
        for folder in cls.label:
            count = 0
            print(f'Press [b] on the keyboard to start data collection of image type: [{folder}]')
            userinput = input()
            if userinput != 'b':
                print("Wrong Input...press 'b'")
                exit()
            while count < cls.num:
                status, frame = camera.read()
                if not status:
                    print("frame doesn't been captured")
                    break
                cv2.startWindowThread()
                cv2.imshow(f"Camera View for image type [{folder}]", frame)
                frame = cv2.resize(frame, (cls.width, cls.height))
                cv2.imwrite(folder+'/'+ str(cls.label[i]) + str(count) + '.png', frame)
                count=count+1
                time.sleep(cls.standby_time)
                if cv2.waitKey(1) == ord('q'):
                    break
            i += 1
        camera.release()
        cv2.destroyAllWindows()
        cv2.waitKey(1)
        # Set status 
        cls._statusGray = 0
        cls._statusRGB = 1
    

    @classmethod
    def compress_train_test(cls) -> None:
        """
        Saves the images shot in datasets divided by train and test like the mnist dataset.
    
        Raises:
            ValueError: If both rgb and gray functions have not been called before compressing a dataset.
            ValueError: If the percentage value for images in the test set is less than or equal to 0.
    
        Returns:
            None
        """
        # data control
        if cls._statusRGB == 0 and cls._statusGray == 0:
            raise ValueError('You have to call rgb or gray function before compress a dataset...')
        print('\n')
        print('--- DATASET SETTING ---')
        cls.perc = float(input('percentage of images in the test set (0,1): '))
        if cls.perc <= 0:
            raise ValueError('percentage value must be greater than 0...')
        # index for image type 
        i = 0
        # X
        if cls._statusGray == 1:
            cls._tensor['X'] = np.empty((0,cls.height,cls.width))
        else:
            cls._tensor['X'] = np.empty((0,cls.height,cls.width,3))
        # y 
        cls._tensor['y'] = np.empty((0))
        # (append) loop 
        for lab in cls.label:
            j = 0
            if cls._statusGray == 1:
                cls._class_dict['t'+str(i)] = np.empty((cls.num, cls.height, cls.width))
            else:
                cls._class_dict['t'+str(i)] = np.empty((cls.num, cls.height, cls.width, 3))
            # loop for convert image format  
            for file in os.listdir(lab):
                if cls._statusGray == 1:
                    img = cv2.imread(lab + '/' + file, cv2.IMREAD_GRAYSCALE)
                else:
                    img = cv2.imread(lab + '/' + file)
                    img = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
                # save
                cls._class_dict['t'+str(i)][j] = img
                j += 1
            # unique final _tensors 
            cls._tensor['X'] = np.append(cls._tensor['X'], cls._class_dict['t'+str(i)], axis = 0)
            cls._tensor['y'] = np.append(cls._tensor['y'], np.repeat(i, cls.num, axis = 0))
            i += 1
        # create dataset (mnist style)
        cls._tensor['X'] = cls._tensor['X'].astype('uint8')
        cls._tensor['y'] = cls._tensor['y'].astype('uint8')
        cls.X_train, cls.X_test, cls.y_train, cls.y_test = train_test_split(cls._tensor['X'], cls._tensor['y'], test_size=cls.perc, random_state=123)
        np.savez('dataset.npz', X_train=cls.X_train, X_test=cls.X_test, y_train=cls.y_train, y_test=cls.y_test)


    @classmethod
    def compress_all(cls) -> None:
        """
        Saves the images shot in a unique dataset.
    
        Raises:
            ValueError: If both rgb and gray functions have not been called before compressing a dataset.
        
        Returns:
            None
        """
        # data control
        if cls._statusRGB == 0 and cls._statusGray == 0:
            raise ValueError('You have to call rgb or gray function before compress a dataset...')
        # index for image type 
        i = 0
        # X
        if cls._statusGray == 1:
            cls._tensor['X'] = np.empty((0,cls.height,cls.width))
        else:
            cls._tensor['X'] = np.empty((0,cls.height,cls.width,3))
        # y 
        cls._tensor['y'] = np.empty((0))
        # (append) loop 
        for lab in cls.label:
            j = 0
            if cls._statusGray == 1:
                cls._class_dict['t'+str(i)] = np.empty((cls.num, cls.height, cls.width))
            else:
                cls._class_dict['t'+str(i)] = np.empty((cls.num, cls.height, cls.width, 3))
            # loop for convert image format
            for file in os.listdir(lab):
                if cls._statusGray == 1:
                    img = cv2.imread(lab + '/' + file, cv2.IMREAD_GRAYSCALE)
                else:
                    img = cv2.imread(lab + '/' + file)
                    img = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
                # save 
                cls._class_dict['t'+str(i)][j] = img
                j += 1
            # unique final _tensors 
            cls._tensor['X'] = np.append(cls._tensor['X'], cls._class_dict['t'+str(i)], axis = 0)
            cls._tensor['y'] = np.append(cls._tensor['y'], np.repeat(i, cls.num, axis = 0))
            i += 1
        # create dataset (mnist style)
        cls._tensor['X'] = cls._tensor['X'].astype('uint8')
        cls._tensor['y'] = cls._tensor['y'].astype('uint8')
        np.savez('datasetall.npz', x = cls._tensor['X'], y = cls._tensor['y'])


    @classmethod
    def just_compress(cls) -> None:
        """
        Saves the images shot in a unique dataset without saving the y variable containing the type of the single image.
    
        Raises:
            ValueError: If both rgb and gray functions have not been called before compressing a dataset.
        
        Returns:
            None
        """
        # data control
        if cls._statusRGB == 0 and cls._statusGray == 0:
            raise ValueError('You have to call rgb or gray function before compress a dataset...')
        # insert name for the compress file 
        print('\n')
        print('--- DATASET SETTING ---')
        name = input('Select a name for the compressed file: ')
        # check the name for the dataset 
        if len(name) == 0:
            raise ValueError("Insert a valide name for the compress file...")
        if name == "0":
            pass
        else:
            cls.name = name 
        # index for image type 
        i = 0
        # X
        if cls._statusGray == 1:
            cls._tensor['X'] = np.empty((0,cls.height,cls.width))
        else:
            cls._tensor['X'] = np.empty((0,cls.height,cls.width,3))
        # (append) loop 
        for lab in cls.label:
            j = 0
            if cls._statusGray == 1:
                cls._class_dict['t'+str(i)] = np.empty((cls.num, cls.height, cls.width))
            else:
                cls._class_dict['t'+str(i)] = np.empty((cls.num, cls.height, cls.width, 3))
            # loop for convert image format
            for file in os.listdir(lab):
                if cls._statusGray == 1:
                    img = cv2.imread(lab + '/' + file, cv2.IMREAD_GRAYSCALE)
                else:
                    img = cv2.imread(lab + '/' + file)
                    img = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
                # save
                cls._class_dict['t'+str(i)][j] = img
                j += 1
            # unique final _tensors
            cls._tensor['X'] = np.append(cls._tensor['X'], cls._class_dict['t'+str(i)], axis = 0)
            i += 1
        # create dataset (mnist style)
        cls._tensor['X'] = cls._tensor['X'].astype('uint8')
        np.savez(f'{cls.name}.npz', x = cls._tensor['X'])


    @classmethod
    def var_control(cls) -> None:
        """
        Print the parameters specified in the setup method about the dataset to create.
        """
        print('\n')
        print('--- PARAMETERS CONTROL ---')
        print(f'Camera index: {cls.index}')
        print(f'Labels of images types: {cls.label}')
        print(f'Num. of images for types: {cls.num}')
        print(f'Single frame HEIGHT: {cls.height}')
        print(f'Single frame WIDTH: {cls.width}')
        print(f'Waiting time between frames: {cls.standby_time}')
        print(f'Percentage of images in train dataset: {cls.perc}')
        print(f'StatusGray: {cls._statusGray}')
        print(f'StatusRGB: {cls._statusRGB}')



import cv2 
from sklearn.model_selection import train_test_split
import numpy as np
from typing import List, Tuple


class BuildDataset:
    """
    Builds a dataset by merging multiple datasets with the given parameters.

    Attributes:
        path (List[str]): List of paths to the numpy files containing the datasets.
        label (List[int]): List of labels corresponding to each dataset.
        size (tuple): Tuple specifying the size of the images in the dataset. Defaults to None.
        color (bool): Whether the images are in color or grayscale. Defaults to True.
        split (bool): Whether to split the dataset into train and test sets. Defaults to True.
        perc (float): The percentage of data to use for the test set. Defaults to 0.1.
    
    Examples:
        >>> import xeye
        >>> # list of directory (paths for the .npz files)
        >>> path = ['batch_1.npz','batch_2.npz', 'batch_3.npz']
        >>> # list of labels associated with the images inside the .npz files
        >>> label = [0,1,2]
        >>> data = xeye.BuildDataset(path=path, label=label, size = None, color=True, split=True, perc=0.2)
        >>> data.build()
    """
    def __init__(self, path: List[str], label: List[int], size: Tuple = None, color: bool = True, split: bool = True, perc: float = 0.1) -> None:
        self.path = path
        self.label = label 
        self.size = size
        self.color = color 
        self.split = split
        self.perc = perc
        self.height = 0
        self.width = 0 
        self._tensor = {}
        self._temp_tensor = []
    

    def _control(self) -> None:
        """
        Checks if the datasets to merge have the same colour space.

        Raises:
            ValueError: If the datasets have different color spaces.

        Returns:
            None
        
        Notes:
            * This method extracts the height, width, and color channels of each image in the dataset using numpy.
            * It then checks if the images have the same color space by comparing the color channel values for each image.
            * If the sizes of the images in the dataset are not specified in the instance variables, it sets the maximum height and width of all the images as the dataset size.
        """
        height = []
        width = []
        color_ch = []
        for i in range(len(self.path)):
            data = np.load(f'{self.path[i]}')
            x = data['x']
            height.append(x.shape[1]) # height
            width.append(x.shape[2]) # width
            try:
                color_ch.append(x.shape[3]) # color 
            except:
                color_ch.append(1)
        # Size control
        if self.size == None:
            self.size = []
            self.size.append(max(height))
            self.size.append(max(width))
            self.size = tuple(self.size)
        else:
            pass
        # Color channels control
        if len(set(color_ch)) != 1:
            raise ValueError("Datasets with different colour spaces...used datasets with the same colour spaces for the images")
        else:
            pass


    def build(self) -> None:
        """
        Builds a new dataset by merging the datasets with the parameters indicated by the instance variables.

        Raises:
            ValueError: If the path and label lists do not have the same length.
            ValueError: If the datasets being merged have different color spaces.
        
        Returns:
            None

        Note:
            The method calls the `_control` method to check if the datasets being merged have the same color space. The resulting merged dataset is stored as a numpy array in `_tensor['X']` and `_tensor['y']`. If the `split` instance variable is set to `True`, the merged dataset is split into training and testing sets using the `train_test_split` method from scikit-learn and saved as a numpy array in the file 'dataset.npz'. Otherwise, the merged dataset is saved as a numpy array in the file 'datasetall.npz'.
        """
        # control method calling 
        self._control()
        # control if the path and label lists have the same length
        if len(self.path) != len(self.label):
            raise ValueError("Path and label lists doesn't have the same length...")
        # Create the tensor X
        if self.color == True:
            self._tensor['X'] = np.empty((0,self.size[0],self.size[1],3)).astype('uint8')
        else: 
            self._tensor['X'] = np.empty((0,self.size[0],self.size[1])).astype('uint8')
        # array for label y 
        self._tensor['y'] = np.empty((0))
        # loop 
        for i in range(len(self.path)):
            data = np.load(f'{self.path[i]}')
            x = data['x']
            if self.color == True:
                self._temp_tensor  = np.zeros((x.shape[0],self.size[0],self.size[1],3)).astype('uint8')
            else: 
                self._temp_tensor = np.zeros((x.shape[0],self.size[0],self.size[1])).astype('uint8')
                # inner loop for resizing the images 
            for j in range(x.shape[0]):
                new_img = cv2.resize(x[j], (self.size[1],self.size[0])) # (width, height)
                self._temp_tensor[j] = new_img
            # save the resized images 
            self._tensor['X'] = np.concatenate((self._tensor['X'],self._temp_tensor),axis=0)
            self._tensor['y'] = np.append(self._tensor['y'], np.repeat(self.label[i], x.shape[0], axis = 0))
        # create the dataset
        if self.split == True:
            self.X_train, self.X_test, self.y_train, self.y_test = train_test_split(self._tensor['X'], self._tensor['y'], test_size=self.perc, random_state=123)
            np.savez('dataset.npz', X_train=self.X_train, X_test=self.X_test, y_train=self.y_train, y_test=self.y_test)
        else: 
            np.savez('datasetall.npz', x = self._tensor['X'], y = self._tensor['y'])



import cv2 
import os
from sklearn.model_selection import train_test_split
import numpy as np
import time
from typing import List


class FastDataset:
    """
    A class for shooting and saving images in grayscale or RGB using OpenCV.

    Attributes:
        index (int): the index of the camera to use.
        img_types (int): the number of types of images to collect.
        label (List[str]): a list of strings that represent the name of the directories where the images will be saved.
        num (int): the number of frames to capture for each image type.
        height (int): the height of the frames to capture.
        width (int): the width of the frames to capture.
        standby_time (float): the time to wait before capturing each frame. 
    
    Examples: 
        >>> import xeye
        >>> # define parameters values
        >>> index = 0
        >>> img_types = 2
        >>> label = ['keyboard', 'mouse']
        >>> num = 20
        >>> height = 100
        >>> width = 100
        >>> standby_time = 0
        >>> data = xeye.FastDataset(index = index, img_types = img_types, label = label, num = num, height = height, width = width, stand_by_time = standby_time)
        >>> data.preview()
        >>> data.rgb() # or data.gray()
        >>> data.compress_train_test(perc=0.2)
        >>> data.compress_all()
        >>> data.just_compress(name="batch_test")
    """
    def __init__(self, index: int, img_types: int, label: List[str], num: int, height: int, width: int, stand_by_time: float) -> None:
        self.index = index
        self.img_types = img_types
        self.label = label
        self.num = num
        self.height = height
        self.width = width
        self._class_dict = {}
        self._tensor = {}
        self.standby_time = stand_by_time
        self._statusGray = 0
        self._statusRGB = 0
        # clear terminal 
        if(os.name == 'posix'): # unix
            os.system('clear')
        else: # windows
            os.system('cls')
        # camera setting
        if self.index == -1:
            raise ValueError('(index) Insert valid camera index...')
        camera = cv2.VideoCapture(self.index)
        if camera.isOpened() == False:
            raise ValueError('(index) Insert valid camera index...')
        # set how many type of images do you want to collect
        if self.img_types == 0: 
            raise ValueError('(img_types) Number of images types must be greather than 0')
        if type(self.img_types) != int:
            raise TypeError('(img_types) passed a string, not an integer')
        # folder building
        if self.label == []:
            raise ValueError('(label) Not valid names for images types...')
        if len(self.label) != self.img_types:
            raise ValueError("(label) You must have a number of labels equal to the number of images types selected...")
        for lab in self.label:
            if not os.path.exists(lab):
                os.mkdir(lab)
        # number of frames 
        if self.num == 0 or self.num < 0:
            raise ValueError('(num) You cant inizialize frames number equal or below zero...')
        # Image shaping phase
        if self.height == 0 or self.height < 0:
            raise ValueError('(height) Frame HEIGHT must be greather than 0')
        if self.width == 0 or self.width < 0:
            raise ValueError('(width) Frame WIDTH must be greather than 0')
        # Waiting time in shooting loop
        if self.standby_time < 0:
            raise ValueError('(standby_time) waiting time must be grater than 0...')


    def preview(self) -> None:
        """
        Opens the camera stream on a window for checking the framing of the image.

        Returns: 
            None
        """
        print('\n')
        print('--- PREVIEW ---')
        camera = cv2.VideoCapture(self.index)
        while(True):
            status, frame = camera.read()
            if not status:
                print("frame doesn't been captured")
                break
            font = cv2.FONT_HERSHEY_COMPLEX
            text = 'click on the image window and then press [q] on the keyboard to quit preview'
            cv2.putText(frame,text,(5,50),font,0.8,(124,252,0),2)  #text,coordinate,font,size of text,color,thickness of font
            cv2.startWindowThread()
            cv2.imshow("Camera PreView", frame)
            if cv2.waitKey(1) == ord('q'):
                print('preview closed')
                camera.release()
                cv2.destroyAllWindows()
                cv2.waitKey(1)
                break


    def gray(self) -> None:
        """
        Method for shooting images in grayscale.

        Returns:
            None 
        """
        print('\n')
        print('--- START TAKING PHOTOS ---')
        camera = cv2.VideoCapture(self.index)
        # Index for files name 
        i = 0
        for folder in self.label:
            count = 0
            print(f'Press [b] on the keyboard to start data collection of image type: [{folder}]')
            userinput = input()
            if userinput != 'b':
                print("Wrong Input...press 'b'")
            while count < self.num:
                status, frame = camera.read()
                if not status:
                    print("frame doesn't been captured")
                    break
                gray = cv2.cvtColor(frame, cv2.COLOR_BGR2GRAY)
                cv2.startWindowThread()
                cv2.imshow(f"Camera View for image type [{folder}]",gray)
                gray = cv2.resize(gray, (self.width, self.height))
                cv2.imwrite(folder+'/'+ str(self.label[i]) + str(count) + '.png', gray)
                count=count+1
                time.sleep(self.standby_time)
                if cv2.waitKey(1) == ord('q'):
                    break
            i += 1
        camera.release()
        cv2.destroyAllWindows()
        cv2.waitKey(1)
        # set status
        self._statusGray = 1
        self._statusRGB = 0


    def rgb(self) -> None:
        """
        Method for shooting images in RGB.

        Returns:
            None
        """
        print('\n')
        print('--- START TAKING PHOTOS ---')
        camera = cv2.VideoCapture(self.index)
        # Index for files name 
        i = 0
        for folder in self.label:
            count = 0
            print(f'Press [b] on the keyboard to start data collection of image type: [{folder}]')
            userinput = input()
            if userinput != 'b':
                print("Wrong Input...press 'b'")
                exit()
            while count < self.num:
                status, frame = camera.read()
                if not status:
                    print("frame doesn't been captured")
                    break
                cv2.startWindowThread()
                cv2.imshow(f"Camera View for image type [{folder}]", frame)
                frame = cv2.resize(frame, (self.width, self.height))
                cv2.imwrite(folder+'/'+ str(self.label[i]) + str(count) + '.png', frame)
                count=count+1
                time.sleep(self.standby_time)
                if cv2.waitKey(1) == ord('q'):
                    break
            i += 1
        camera.release()
        cv2.destroyAllWindows()
        cv2.waitKey(1)
        # Set status 
        self._statusGray = 0
        self._statusRGB = 1


    def compress_train_test(self, perc: float = 0.1) -> None:
        """
        Saves the images shot in datasets divided by train and test like the mnist dataset.
    
        Args:
            perc (float): The percentage of images to assign to the test dataset.

        Raises:
            ValueError: If both rgb and gray functions have not been called before compressing a dataset.
            ValueError: If the percentage value for images in the test set is less than or equal to 0.
    
        Returns:
            None
        """
        # percentage control 
        if perc <= 0:
            raise ValueError('(perc) Percentage value must be greater than 0...')
        # data control
        if self._statusRGB == 0 and self._statusGray == 0:
            raise ValueError('You have to call rgb or gray function before compress a dataset...')
        # index for image type 
        i = 0
        # X
        if self._statusGray == 1:
            self._tensor['X'] = np.empty((0,self.height,self.width))
        else:
            self._tensor['X'] = np.empty((0,self.height,self.width,3))
        # y 
        self._tensor['y'] = np.empty((0))
        # (append) loop 
        for lab in self.label:
            j = 0
            if self._statusGray == 1:
                self._class_dict['t'+str(i)] = np.empty((self.num, self.height, self.width))
            else:
                self._class_dict['t'+str(i)] = np.empty((self.num, self.height, self.width, 3))
            # loop for convert image format  
            for file in os.listdir(lab):
                if self._statusGray == 1:
                    img = cv2.imread(lab + '/' + file, cv2.IMREAD_GRAYSCALE)
                else:
                    img = cv2.imread(lab + '/' + file)
                    img = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
                # save in _tensor class 
                self._class_dict['t'+str(i)][j] = img
                j += 1
            # unique final _tensors 
            self._tensor['X'] = np.append(self._tensor['X'], self._class_dict['t'+str(i)], axis = 0)
            self._tensor['y'] = np.append(self._tensor['y'], np.repeat(i, self.num, axis = 0))
            i += 1
        # create dataset (mnist style)
        self._tensor['X'] = self._tensor['X'].astype('uint8')
        self._tensor['y'] = self._tensor['y'].astype('uint8')
        self.X_train, self.X_test, self.y_train, self.y_test = train_test_split(self._tensor['X'], self._tensor['y'], test_size=perc, random_state=123)
        np.savez('dataset.npz', X_train=self.X_train, X_test=self.X_test, y_train=self.y_train, y_test=self.y_test)


    def compress_all(self) -> None:
        """
        Saves the images shot in a unique dataset.
    
        Raises:
            ValueError: If both rgb and gray functions have not been called before compressing a dataset.
        
        Returns:
            None
        """
        # data control
        if self._statusRGB == 0 and self._statusGray == 0:
            raise ValueError('You have to call rgb or gray function before compress a dataset...')
        # index for image type 
        i = 0
        # X
        if self._statusGray == 1:
            self._tensor['X'] = np.empty((0,self.height,self.width))
        else:
            self._tensor['X'] = np.empty((0,self.height,self.width,3))
        # y 
        self._tensor['y'] = np.empty((0))
        # (append) loop 
        for lab in self.label:
            j = 0
            if self._statusGray == 1:
                self._class_dict['t'+str(i)] = np.empty((self.num, self.height, self.width))
            else:
                self._class_dict['t'+str(i)] = np.empty((self.num, self.height, self.width, 3))
            # loop for convert image format
            for file in os.listdir(lab):
                if self._statusGray == 1:
                    img = cv2.imread(lab + '/' + file, cv2.IMREAD_GRAYSCALE)
                else:
                    img = cv2.imread(lab + '/' + file)
                    img = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
                # save in _tensor class 
                self._class_dict['t'+str(i)][j] = img
                j += 1
            # unique final _tensors 
            self._tensor['X'] = np.append(self._tensor['X'], self._class_dict['t'+str(i)], axis = 0)
            self._tensor['y'] = np.append(self._tensor['y'], np.repeat(i, self.num, axis = 0))
            i += 1
        # create dataset (mnist style)
        self._tensor['X'] = self._tensor['X'].astype('uint8')
        self._tensor['y'] = self._tensor['y'].astype('uint8')
        np.savez('datasetall.npz', x = self._tensor['X'], y = self._tensor['y'])


    def just_compress(self, name: str = "dataset_raw") -> None:
        """
        Saves the images shot in a unique dataset without saving the y variable containing the type of the single image.

        Args:
            name (str): The name of the dataset (npz) to be saved.
    
        Raises:
            ValueError: If both rgb and gray functions have not been called before compressing a dataset.
        
        Returns:
            None
        """
        # data control
        if self._statusRGB == 0 and self._statusGray == 0:
            raise ValueError('You have to call rgb or gray function before compress a dataset...')
        # check the name for the dataset 
        if len(str(name)) == 0:
            raise ValueError("name: Insert a valide name for the compressed file...")
        # index for image type 
        i = 0
        # X
        if self._statusGray == 1:
            self._tensor['X'] = np.empty((0,self.height,self.width))
        else:
            self._tensor['X'] = np.empty((0,self.height,self.width,3))
        # (append) loop 
        for lab in self.label:
            j = 0
            if self._statusGray == 1:
                self._class_dict['t'+str(i)] = np.empty((self.num, self.height, self.width))
            else:
                self._class_dict['t'+str(i)] = np.empty((self.num, self.height, self.width, 3))
            # loop for convert image format
            for file in os.listdir(lab):
                if self._statusGray == 1:
                    img = cv2.imread(lab + '/' + file, cv2.IMREAD_GRAYSCALE)
                else:
                    img = cv2.imread(lab + '/' + file)
                    img = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
                # save in _tensor class 
                self._class_dict['t'+str(i)][j] = img
                j += 1
            # unique final _tensors
            self._tensor['X'] = np.append(self._tensor['X'], self._class_dict['t'+str(i)], axis = 0)
            i += 1
        # create dataset (mnist style)
        self._tensor['X'] = self._tensor['X'].astype('uint8')
        np.savez(f'{name}.npz', x = self._tensor['X'])


    def var_control(self) -> None:
        """
        Print the parameters specified in the init method about the dataset to create.
        """
        print('\n')
        print('--- PARAMETERS CONTROL ---')
        print(f'Camera index: {self.index}')
        print(f'Num. of images types: {self.img_types}')
        print(f'Labels of images types: {self.label}')
        print(f'Num. of images for type: {self.num}')
        print(f'Single frame HEIGHT: {self.height}')
        print(f'Single frame WIDTH: {self.width}')
        print(f'Waiting time between frames: {self.standby_time}')
        print(f'StatusGray: {self._statusGray}')
        print(f'StatusRGB: {self._statusRGB}')
